//
// syscall c-style generator
//
 
#include <windows.h>
#include <stdio.h>
 
enum GENERATION_TYPE { G_DEFINES, G_NAME_ARRAY, G_BOTH };
 
#define DEFINE_PREFIX   "Sdt_"
#define ORDINAL_TYPE    "0x%03x"
#define GENERATE_NT     1
#define GENERATE_ZW     1
 
#define ARRAY_NAME  "SdtFunctions"
 
//GENERATION_TYPE GenType = G_NAME_ARRAY;
GENERATION_TYPE GenType = G_BOTH;
 
char *GetWinNameByVer (ULONG Major, ULONG Minor)
{
    switch (Major)
    {
    case 4:
 
        return "NT 4";
 
    case 5:
 
        switch (Minor)
        {
        case 0: return "2000";
        case 1: return "XP";
        case 2: return "2003 Server";
        }
 
    case 6:
 
        switch (Minor)
        {
        case 0: return "Vista";
        }
 
    }
 
    return "(unk)";
}
 
int _main();
 
int main()
{
    OSVERSIONINFOEX ver = {sizeof(ver)};
 
    if (!GetVersionEx ( (OSVERSIONINFO*)&ver ))
        return printf("Cannot ger version\n");
 
    printf("//\n// syscall info\n// generated for Windows %s (NT %d.%d) Build %d %s\n//\n\n",
        GetWinNameByVer (ver.dwMajorVersion, ver.dwMinorVersion),
        ver.dwMajorVersion, ver.dwMinorVersion, ver.dwBuildNumber, ver.szCSDVersion);
 
 
    switch (GenType)
    {
    case G_DEFINES:
    case G_NAME_ARRAY:
        return _main();
 
    case G_BOTH:
        GenType = G_NAME_ARRAY;
        _main();
 
        GenType = G_DEFINES;
        _main();
    }
 
    return 0;
}
 
int _main()
{
    HMODULE Base = GetModuleHandle ("ntdll.dll");
    if (!Base)
        return printf("Cannot get handle of ntdll\n");
 
    PIMAGE_DOS_HEADER mz;
    PIMAGE_FILE_HEADER pfh;
    PIMAGE_OPTIONAL_HEADER poh;
    PIMAGE_EXPORT_DIRECTORY pexd;
    PDWORD AddressOfFunctions;
    PDWORD AddressOfNames;
    PWORD AddressOfNameOrdinals;
    int i;
    
    // Get headers
    *(PBYTE*)&mz = (PBYTE)Base;
    *(PBYTE*)&pfh = (PBYTE)Base + mz->e_lfanew + sizeof(IMAGE_NT_SIGNATURE);
    *(PIMAGE_FILE_HEADER*)&poh = pfh + 1;
 
    // Get export
    *(PBYTE*)&pexd = (PBYTE)Base + poh->DataDirectory[IMAGE_DIRECTORY_ENTRY_EXPORT].VirtualAddress;
 
    *(PBYTE*)&AddressOfFunctions = (PBYTE)Base + pexd->AddressOfFunctions;
    *(PBYTE*)&AddressOfNames = (PBYTE)Base + pexd->AddressOfNames;
    *(PBYTE*)&AddressOfNameOrdinals = (PBYTE)Base + pexd->AddressOfNameOrdinals;
 
    PCHAR SdtFunctions [0x400] = {0};
    ULONG Number = 0;
 
    for( i=0; i<pexd->NumberOfNames; i++ )
    {
        PCHAR Name = ((char*)Base + AddressOfNames[i]);
        PVOID Addr = (PVOID*)((DWORD)Base + AddressOfFunctions[AddressOfNameOrdinals[i]]);
 
        if ( *(USHORT*)Name == 'tN' ) // Nt**
        {
            if ( *(UCHAR*)Addr == 0xB8 ) // MOV EAX, XXXXXXXX
            {
                Number = *(ULONG*)( (UCHAR*)Addr + 1 );
 
                switch (GenType)
                {
                case G_DEFINES:
                    
#if GENERATE_NT
                    printf ("#define  " DEFINE_PREFIX "%-52s" ORDINAL_TYPE "\n", Name, Number);
#endif
 
#if GENERATE_ZW
                    printf ("#define  " DEFINE_PREFIX "Zw%-50s" ORDINAL_TYPE "\n", Name+2, Number);
#endif
 
                    break;
 
                case G_NAME_ARRAY:
 
                    SdtFunctions [Number] = Name;
                    break;
                }
            }
        }
    }
 
    if (GenType == G_DEFINES)
        return printf("\n");
 
    printf("char* " ARRAY_NAME " [%d] = {\n", Number);
 
    // Name array
    for (i=0; i<=Number; i++)
    {
        char end_char =  (i == Number ? ' ' : ',');
 
        if (SdtFunctions[i] == NULL)
            printf (" NULL%c\n", end_char);
        else
            printf(" \"%s\"%c\n", SdtFunctions[i], end_char);
    }
 
    printf("};\n\n");
 
    return 0;
}